#!/usr/bin/env node
'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const restify = require('restify');
const yargs = require('yargs');
const files = require('../lib/files');
const logging = require('../lib/logger');
const program = require('../lib/program');
const Service = require('../lib/service');
const uuid = require('uuid');
const enableDestroy = require('server-destroy');
const Promise = require('bluebird');

const logger = logging.createLogger('nodus-service');

// Program Help
yargs
    .usage('Usage: $0 <service> [args] [options]')
    .array('plugin')
    .describe('plugin', 'Load a service plugin.')
    .options('name', {required: true})
    .describe('name', 'The name of the service.')
    .describe('path', 'The path of the service to listen for requests.')
    .number('port')
    .alias('p', 'port')
    .default('p', 8080)
    .describe('port', 'Port used to listen for incomming requests.')
    .version()
    .epilog('Copyright (c) 2017 Brad Serbu <bradserbu@bradserbu.com>\nAll rights reserved.')
    .help();

/**
 * Load a service
 * @param service
 */
function loadService(name, targetFile) {
    // Load the module/file to be run
    const target = files.requireFile(targetFile);
    logger.debug('TARGET', target);

    // Create new service
    const service = Service(name, target);
    service.path = service.path || `/${service.name}`;
    service.method = service.method || `GET`;

    return service;
}

// Parse CommandLine Arguments
const argv = yargs.argv;
logger.debug('ARGV', argv);

// Determine the file/module to be run
const targetFile = argv._.shift();
logger.debug('TARGET_FILE', targetFile);

// Display help if no target is specified
if (util.isNullOrUndefined(targetFile)) {
    return yargs.showHelp();
}

// Create a new server
const server = restify.createServer({
    name: 'nodus-service'
});

// Apply Plugins
server.use(restify.acceptParser(server.acceptable));
server.use(restify.dateParser());
server.use(restify.queryParser());
server.use(restify.bodyParser());
server.use(restify.gzipResponse());

// Load the service
const name = argv.name;
const service = loadService(name, targetFile);
logger.debug('SERVICE', service);

// Apply service plugins
function applyPlugins(plugins) {
    return Promise.mapSeries(plugins, name => {
        // Apply service plugin
        logger.debug('APPLY_PLUGIN', {name: name});

        const plugin = require(`../plugins/${name}`);
        if (!plugin)
            throw errors('NOT_FOUND', 'The plugin was not found.', {name: name});

        return plugin(server, service);
    });
}

// ENDPOINT: Service request
const method = service.method;
const path = service.path;
const endpoint = server[method.toLowerCase()].bind(server);
if (!endpoint)
    throw errors('NOT_SUPPORTED', 'The endpoint method is not supported.', {method: method});

endpoint(path, (req, res) => {

    // Generate a new request id
    const request = {
        id: uuid.v4(),
        args: req.params
    };

    logger.debug('HANDLE_REQUEST', request);

    // Execute the service
    service
        .handle(request)
        .then(response => {
            res.send(response);
        })
        .catch(err => {
            logger.debug(err);
            res.send(500, err);
        });
});

enableDestroy(server);

// Shutdown server on program shutdown
program.on('shutdown', () => {
    logger.debug('Shutting down http server...');
    server.destroy();
});

// Apply PLUGINS
const plugins = argv.plugin;
const port = argv.port;

// Apply plugins then start the server
applyPlugins(plugins)
    .then(() => {
        logger.debug('START_SERVICE', {name: service.name});
        return service.start();
    })
    .then(() => {
        // ** Stop service on shutdown
        program.on('shutdown', () => {
            logger.debug('STOP_SERVICE', {name: service.name});
            service.stop();
        });
    })
    .then(() => {
        logger.debug('START_HTTP_LISTENER', {port: port});

        // Start HTTP Server Listener
        server.listen(port);
    });