/**
 * Plugin to log service requests in influxdb
 */

'use strict';

// ** Constants
const DEFAULT_HOST = 'localhost';

// ** Dependencies
const Influx = require('influx');
const os = require('os');
const errors = require('../lib/errors');
const logger = require('../lib/logger').createLogger('influx-plugin');

function loadPlugin(server, service) {

    // Name of the influx database to use
    const host = process.env['INFLUX_HOST'] || DEFAULT_HOST;
    const database = `${service.name}_db`;
    const hostname = os.hostname();

    /**
     * Create a new Influx client. We tell it to use the
     * `express_response_db` database by default, and give
     * it some information about the schema we're writing.
     */
    const influx = new Influx.InfluxDB({
        host: host,
        database: database,
        schema: [
            {
                measurement: 'requests',
                fields: {
                    request_id: Influx.FieldType.STRING,
                    request_args: Influx.FieldType.STRING
                },
                tags: [
                    'hostname'
                ]
            },
            {
                measurement: 'events',
                fields: {
                    event: Influx.FieldType.STRING
                },
                tags: [
                    'hostname'
                ]
            }
        ]
    });

    // Log all request to influx db
    service.on('request', request => {

        // Capture ID of request
        const request_id = request.id;
        const request_args = JSON.stringify(request.args);

        const points = [{
            measurement: 'requests',
            tags: {hostname: hostname},
            fields: {
                request_id: request_id,
                request_args: request_args
            }
        }];

        logger.debug('WRITE_REQUEST', points);
        return influx
            .writePoints(points)
            .catch(err => {
                logger.debug(errors('INFLUX_WRITE_ERROR', 'An error occurred writing to influxdb.', err));
            });
    });

    function captureEvent(event) {

        service.on(event, () => {
            const points = [{
                measurement: 'events',
                tags: {hostname: hostname},
                fields: {
                    event: event
                }
            }];

            logger.debug('WRITE_EVENT', points);
            return influx
                .writePoints(points)
                .catch(err => {
                    logger.debug(errors('INFLUX_WRITE_ERROR', 'An error occurred writing to influxdb.', err));
                });
        });
    }

    captureEvent('started');
    captureEvent('stopped');

    function query_endpoint(path, query) {
        // Add endpoint to get all requests
        server.get(`${service.path}/${path}`, (req, res) => {
            influx.query(query)
                .then(result => res.send(result))
                .catch(err => {
                    res.status(500, err);
                });
        });
    }

    query_endpoint('/requests', `
            select * from requests
            where hostname = ${Influx.escape.stringLit(os.hostname())}
            order by time desc
            limit 100`);

    query_endpoint('/events', `
            select * from events
            where hostname = ${Influx.escape.stringLit(os.hostname())}
            order by time desc
            limit 100`);

    /**
     * Now, we'll make sure the database exists and boot the app.
     */
    return influx
        .getDatabaseNames()
        .then(names => {
            if (!names.includes(database)) {
                return influx.createDatabase(database)
            }
        });
}

// ** Exports
module.exports = loadPlugin;